
// --------------------------------
// File    : ClientLineSystem.cpp
// Function: Client Line System class for the Line System effect
// --------------------------------


#include "de_client.h"
#include "ClientLineSystem.h"
#include "windows.h"
#include "winbase.h"


ClientLineSystem::ClientLineSystem() : Effect() 
{
	VEC_INIT(m_vStart);
	VEC_INIT(m_vEnd);
	VEC_SET(m_vColor1, 255.0f, 255.0f, 255.0f);
}

// ----------------
// Init
//		Initialize the Client Line System
// ----------------
DBOOL ClientLineSystem::Init(PlayerClient *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMessage)
{
	// for later use in the Update Method
	m_StartDeltaX = m_StartDeltaY = m_StartDeltaZ = 0.0f;
	m_EndDeltaX = m_EndDeltaY = m_EndDeltaZ = 0.0f;
	StartDirection = 1;
	EndDirection = -1;
	m_StartAlphaDirection = -0.01f;
	m_EndAlphaDirection = 0.02f;


	m_pPlayer = pPlayer;
	m_hServerObject = hObj;

	// Read our configuration from the message
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vStart);
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vEnd);
	m_pPlayer->GetClient()->ReadFromMessageVector(hMessage,&m_vColor1);


	return CreateLineSystemObject(m_pPlayer->GetClient());
}

// ----------------
// Destroy
//		Destroys the Client Line System
// ----------------
void ClientLineSystem::Destroy()
{
	if (m_pPlayer->GetClient()) 
	{
		m_pPlayer->GetClient()->DeleteObject(m_hObject);
	}
}

// ----------------
// CreateLineSystemObject
//		Create the actual Line System
// ----------------
DBOOL ClientLineSystem::CreateLineSystemObject(ClientDE *pClientDE)
{
    DELine NewLine;
	// Sanity check!
	if (!pClientDE) return DFALSE;

	// Setup the Line System.

	ObjectCreateStruct createStruct;
	INIT_OBJECTCREATESTRUCT(createStruct);

	createStruct.m_ObjectType = OT_LINESYSTEM;
	createStruct.m_Flags = FLAG_VISIBLE|FLAG_SOLID;

	m_hObject = pClientDE->CreateObject(&createStruct);

	pClientDE->GetObjectPos(m_hServerObject,&m_vPos);
	pClientDE->SetObjectPos(m_hObject,&m_vPos);

	// Set the info pertaining to the line we are going to create
	// First, the Start Point
	NewLine.m_Points[0].m_Pos.x = m_vStart.x-m_vPos.x;
	NewLine.m_Points[0].m_Pos.y = m_vStart.y-m_vPos.y;
	NewLine.m_Points[0].m_Pos.z = m_vStart.z-m_vPos.z;

	// Then the end point
	NewLine.m_Points[1].m_Pos.x = m_vEnd.x-m_vPos.x;
	NewLine.m_Points[1].m_Pos.y = m_vEnd.y-m_vPos.y;
	NewLine.m_Points[1].m_Pos.z = m_vEnd.z-m_vPos.z;

	// Finally the Line color
    NewLine.m_Points[0].r = NewLine.m_Points[1].r = m_vColor1.x/255.0f;
	NewLine.m_Points[0].g = NewLine.m_Points[1].g = m_vColor1.y/255.0f;
    NewLine.m_Points[0].b = NewLine.m_Points[1].b = m_vColor1.z/255.0f;
    NewLine.m_Points[0].a = NewLine.m_Points[1].a = 0.10f;
                       
	pClientDE->AddLine(m_hObject, &NewLine);




	return DTRUE;
}

// ----------------
// Update
//		Updates the Line System effect
// ----------------
DBOOL ClientLineSystem::Update()
{
	HDELINE  hCurrentLine, hPrev;
    DELine   LineInfo;

	if(!m_hObject || !m_pPlayer->GetClient()) return DFALSE;


	hPrev = DNULL;
	hCurrentLine = m_pPlayer->GetClient()->GetNextLine(m_hObject, hPrev);

	while (hCurrentLine)
	{
		// Get the line information
		m_pPlayer->GetClient()->GetLineInfo(hCurrentLine, &LineInfo);

		// Modify the Line information:

		if (m_StartDeltaX > 5)
			StartDirection = -1;

		if (m_StartDeltaX < -5)
			StartDirection = 1;

		m_StartDeltaX += StartDirection;
		m_StartDeltaY += StartDirection;
		m_StartDeltaZ += StartDirection;

		if (m_EndDeltaX > 5)
			EndDirection = -1;

		if (m_EndDeltaX < -5)
			EndDirection = 1;

		m_EndDeltaX += EndDirection;
		m_EndDeltaY += EndDirection;
		m_EndDeltaZ += EndDirection;


		LineInfo.m_Points[0].m_Pos.x += m_StartDeltaX;
		LineInfo.m_Points[0].m_Pos.y += m_StartDeltaY;
		LineInfo.m_Points[0].m_Pos.z += m_StartDeltaZ;

		// Then the end point
		LineInfo.m_Points[1].m_Pos.x += m_EndDeltaX;
		LineInfo.m_Points[1].m_Pos.y += m_EndDeltaY;
		LineInfo.m_Points[1].m_Pos.z += m_EndDeltaZ;

		if (LineInfo.m_Points[0].a < 0.001)
			m_StartAlphaDirection = 0.01f;
		if (LineInfo.m_Points[0].a > 0.999)
			m_StartAlphaDirection = -0.01f;

		// Finally the Line color
		LineInfo.m_Points[0].a += m_StartAlphaDirection;

		if (LineInfo.m_Points[1].a < 0.001)
			m_EndAlphaDirection = 0.02f;
		if (LineInfo.m_Points[1].a > 0.999)
			m_EndAlphaDirection = -0.02f;

		LineInfo.m_Points[1].a += m_EndAlphaDirection;

		// And then set the line information again
		m_pPlayer->GetClient()->SetLineInfo(hCurrentLine, &LineInfo);

		hPrev = hCurrentLine;
		hCurrentLine = m_pPlayer->GetClient()->GetNextLine(m_hObject, hPrev);
	}

	return DTRUE;
}


