
// --------------------------------
// File    : ClientParticleSystem.cpp
// Function: Class for the Torch client-side special effect.
//         : Based on Darren Reid's extensions to the original TorchFX demo.
// --------------------------------

#include "de_client.h"
#include "ClientParticleSystem.h"

DBOOL ClientParticleSystem::Init(PlayerClient *pPlayer, HLOCALOBJ hObj, DBYTE byType, HMESSAGEREAD hMessage)
{
    DVector     pos;
    float       LightRadius, LightDuration;

    m_pPlayer = pPlayer;

    m_pPlayer->GetClient()->GetObjectPos(hObj, &pos);

    // Unpack the message.
    LightRadius = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);
    LightDuration = m_pPlayer->GetClient()->ReadFromMessageFloat(hMessage);

	m_bInited = DFALSE;

    StartSystem(m_pPlayer->GetClient(), &pos, LightRadius, LightDuration);

    return DTRUE;
}


void ClientParticleSystem::StartSystem(	ClientDE *pClient, 
										DVector *mainpos, 
										float lightradius, 
										float lightduration)
{
    ObjectCreateStruct          createStruct;
    DVector                     color, pos, vel;
    int                         nParticles;
	DFLOAT						randNum;

    m_pClient = pClient;

    if (!m_bInited) 
    {

        // Create our particle system
        INIT_OBJECTCREATESTRUCT(createStruct);
        createStruct.m_Flags = FLAG_VISIBLE;
        createStruct.m_ObjectType = OT_PARTICLESYSTEM;
        m_hParticles = m_pClient->CreateObject(&createStruct);

        // Set the position
        m_pClient->SetObjectPos(m_hParticles, mainpos);

        m_pClient->SetupParticleSystem(m_hParticles, "textures\\flare1.dtx", 0.0f, PS_BOUNCE, 3);

        // Set up our various properties
        m_LightRadius = lightradius;
        m_LightDuration = lightduration;
        
        // Build initial system
        nParticles = 20;
        m_bInited = 1;
    
        for(int i=0; i < nParticles; i++)
        {
            randNum = (float)rand() / RAND_MAX;
            color.x = (127.0f * randNum) + 127.0f;
            randNum = (float)rand() / RAND_MAX;
            color.y = (127.0f * randNum) + 127.0f;
            color.z = 0.0f;

            randNum = (float)rand() / RAND_MAX;
            pos.x = (randNum * 10.0f) - 5.0f;
            randNum = (float)rand() / RAND_MAX;
            pos.z = (randNum * 10.0f) - 5.0f;
            pos.y = 0;

            randNum = (float)rand() / RAND_MAX;
            vel.y = (randNum * 10.0f) + 25.0f;
            randNum = (float)rand() / RAND_MAX;
            vel.x = (randNum * 1.5f);
            randNum = (float)rand() / RAND_MAX;
            vel.z = (randNum * 1.5f);

            randNum = (float)rand() / RAND_MAX;
        
            m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 1.0f) + 0.2f);
        }

        // Create our lightsource
        INIT_OBJECTCREATESTRUCT(createStruct);
        createStruct.m_Flags = FLAG_VISIBLE | FLAG_SOLID;
        createStruct.m_ObjectType = OT_LIGHT;
        createStruct.m_Pos = *mainpos;

        m_hLight = m_pClient->CreateObject(&createStruct);
        m_pClient->SetLightColor(m_hLight,1.0f,1.0f,0.0f);
        m_pClient->SetLightRadius(m_hLight, lightradius);
    }
}

DBOOL ClientParticleSystem::Update()
{
    // Sanity check
    if (m_bInited)
    {
        // Increment our cycle counter
        m_fCycle = m_fCycle + 1.0f; 

        // If we've gone through one light duration, flicker the light and add more particles
        if (m_fCycle > m_LightDuration) 
        {
            DVector                     color, pos, vel;
            int                         nParticles;
            float randNum, randNum2, temp;

            temp = m_LightRadius / 3;

            // Create our flicker color
            randNum = (float)rand() / RAND_MAX;
            randNum = (0.5f * randNum) + 0.5f;
            randNum2 = (float)rand() / RAND_MAX;
            randNum2 = (0.5f * randNum2) + 0.5f;
            m_pClient->SetLightColor(m_hLight,randNum,randNum2,0.0f);           

            // And flicker radius
            randNum = (float)rand() / RAND_MAX;
            randNum = (temp * randNum) + (temp * 2);
            m_pClient->SetLightRadius(m_hLight,randNum);                        

            nParticles = 8;

            for (int i = 0; i < nParticles; i++)
            {
                if(i%2)
                {
                    randNum = (float)rand() / RAND_MAX;
                    color.x = (127.0f * randNum) + 127.0f;
                    randNum = (float)rand() / RAND_MAX;
                    color.y = (127.0f * randNum) + 127.0f;
                    color.z = 0.0f;

                    randNum = (float)rand() / RAND_MAX;
                    pos.x = (randNum * 10.0f) - 5.0f;
                    randNum = (float)rand() / RAND_MAX;
                    pos.z = (randNum * 10.0f) - 5.0f;
                    pos.y = 4.0f;

                    randNum = (float)rand() / RAND_MAX;
                    vel.y = (randNum * 10.0f) + 25.0f;
                    randNum = (float)rand() / RAND_MAX;
                    vel.x = (randNum * 1.5f);
                    randNum = (float)rand() / RAND_MAX;
                    vel.z = (randNum * 1.5f);

                    randNum = (float)rand() / RAND_MAX;
        
                    m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.9f) + 0.2f);
                }
                else if(!(i%8))
                {
                    randNum = (float)rand() / RAND_MAX;
                    color.x = (10.0f * randNum);
                    randNum = (float)rand() / RAND_MAX;
                    color.y = (10.0f * randNum);
                    randNum = (float)rand() / RAND_MAX;
                    color.z = (150.0f * randNum) + 104.0f;
    
                    randNum = (float)rand() / RAND_MAX;
                    pos.x = (randNum * 6.0f) - 3.0f;
                    randNum = (float)rand() / RAND_MAX;
                    pos.z = (randNum * 6.0f) - 3.0f;
                    pos.y = 0;

                    randNum = (float)rand() / RAND_MAX;
                    vel.y = (randNum * 10.0f) + 20.0f;
                    randNum = (float)rand() / RAND_MAX;
                    vel.x = (randNum * 1.5f);
                    randNum = (float)rand() / RAND_MAX;
                    vel.z = (randNum * 1.5f);

                    randNum = (float)rand() / RAND_MAX;
        
                    m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.2f) + 0.2f);
                }
                else
                {
                    randNum = (float)rand() / RAND_MAX;
                    color.x = (100.0f * randNum) + 154.0f;
                    randNum = (float)rand() / RAND_MAX;
                    color.y = (50.0f * randNum) + 50.0f;
                    color.z = 0.0f;
    
                    randNum = (float)rand() / RAND_MAX;
                    pos.x = (randNum * 8.0f) - 4.0f;
                    randNum = (float)rand() / RAND_MAX;
                    pos.z = (randNum * 8.0f) - 4.0f;
                    pos.y = 1.0f;

                    randNum = (float)rand() / RAND_MAX;
                    vel.y = (randNum * 10.0f) + 15.0f;
                    randNum = (float)rand() / RAND_MAX;
                    vel.x = (randNum * 1.5f);
                    randNum = (float)rand() / RAND_MAX;
                    vel.z = (randNum * 1.5f);

                    randNum = (float)rand() / RAND_MAX;
        
                    m_pClient->AddParticle(m_hParticles, &pos, &vel, &color, (randNum * 0.6f) + 0.2f);
                }
            }
        }
	}
  return DTRUE;
}

void ClientParticleSystem :: DestroySystem ( )
{
    if (m_bInited) {
        m_pClient->DeleteObject(m_hParticles);
        m_pClient->DeleteObject(m_hLight);
    }
}
