// --------------------------------
// File    : EffectsManager.cpp
// Function: Handles the various special effects; adding, removing and updating them
//         : as appropriate.
// --------------------------------

#include "de_client.h"
#include "effectsmanager.h"
#include "messageIDs.h"

#include "windows.h"
#include "winbase.h"

// Include Effect files here
#include "PolyGrid.h"
#include "ClientParticleSystem.h"
#include "ClientLineSystem.h"

// ----------------
// Handler
//      Receives SpecialEffects messages and dispatches them to the correct effects.
// ----------------
void EffectsManager::Handler(HLOCALOBJ hObj, HMESSAGEREAD hMsg)
{
    if (!m_pPlayer) return;

    DBYTE byType;

    // Read the message type from our message.
    byType = m_pPlayer->GetClient()->ReadFromMessageByte(hMsg);

    // Which type is it...
    switch(byType)
    {
        case MSG_ADD_POLYGRID:
            // It's a polygrid
            PolyGrid *pGrid;

            pGrid = new PolyGrid();
            if (pGrid->Init(m_pPlayer,hObj,byType,hMsg))
                AddEffect(pGrid);
            else
                delete pGrid;

            break;

        case MSG_ADD_PARTICLESYSTEM:
            // It's a torch
            ClientParticleSystem *pParticleSystem;

            pParticleSystem = new ClientParticleSystem();
            if (pParticleSystem->Init(m_pPlayer,hObj,byType,hMsg))
                AddEffect(pParticleSystem);
            else
                delete pParticleSystem;

            break;

        case MSG_ADD_LINESYSTEM:
            // It's a torch
            ClientLineSystem *pLineSystem;

            pLineSystem = new ClientLineSystem();
            if (pLineSystem->Init(m_pPlayer,hObj,byType,hMsg))
                AddEffect(pLineSystem);
            else
                delete pLineSystem;

            break;

        default:
            // We don't know what it is.
            m_pPlayer->GetClient()->CPrint("EffectManager: unknown effect %d", byType);
            break;
    }
}

// ----------------
// Update
//      Updates all active effects
// ----------------
void EffectsManager::Update()
{
    Effect *pEffect, *pTemp;

    // Get the head of our effect list.
    pEffect = m_pEffects;

    // Walk the list
    while (pEffect)
    {
        pTemp = pEffect->GetNext();

        // Perform the update
        if (!pEffect->Update())
            DelEffect(pEffect);

        pEffect = pTemp;
    }
}

// ----------------
// AddEffect
//      Adds an Effect to the Effect list.
// ----------------
void EffectsManager::AddEffect(Effect *pEffect)
{
    pEffect->SetNext(m_pEffects);
    pEffect->SetPrev(NULL);

    if (m_pEffects)
        m_pEffects->SetPrev(pEffect);

    m_pEffects = pEffect;
}

// ----------------
// DelEffect
//      Deletes an Effect from the Effect list.
// ----------------
void EffectsManager::DelEffect(Effect *pEffect)
{
    // Make sure our list is preserved intact
    if (pEffect->GetNext())
        pEffect->GetNext()->SetPrev(pEffect->GetPrev());

    if (pEffect->GetPrev())
        pEffect->GetPrev()->SetNext(pEffect->GetNext());
    else
        m_pEffects = pEffect->GetNext();

    // Allow the effect to clean up...
    pEffect->Destroy();

    // And delete the pointer
    delete pEffect;
}

// ----------------
// DestroyAll
//      Destroys the entire Effects list.
// ----------------
void EffectsManager::DestroyAll()
{
    Effect *pEffect, *pTemp;

    pEffect = m_pEffects;

    while (pEffect)
    {
        pTemp = pEffect->GetNext();

        pEffect->Destroy();
        delete pEffect;

        pEffect = pTemp;
    }
	m_pEffects = NULL;
}
