#include "de_server.h"
#include "DarkClaw.h"



DDWORD DarkClaw::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
    float moveSpeed = 45.0f;
    float terminalVelocity = 45.0f;
    DVector velocity, forward, right, up, position, temp;
    DRotation rotation;
    ObjectCreateStruct *pObjectCreateStruct;
    DVector dims;


    switch(messageID)
    {
        case MID_PRECREATE:
        {
            pObjectCreateStruct = (ObjectCreateStruct*)pData;
            
            // The bad-guy character should have close to the same attibutes as we do.
            pObjectCreateStruct->m_ObjectType = OT_MODEL;
            pObjectCreateStruct->m_Flags |= FLAG_SOLID | FLAG_GRAVITY | FLAG_VISIBLE | FLAG_TOUCH_NOTIFY | FLAG_SHADOW | FLAG_STAIRSTEP | FLAG_FULLPOSITIONRES;

            // Now we load the model we want to use.
            strcpy(pObjectCreateStruct->m_Filename, "models\\claw.abc");
            strcpy(pObjectCreateStruct->m_SkinName, "textures\\DarkClaw.dtx");
			m_bPlayerInAir = false;

            m_ActionState	= STAND;
			m_Pitch			= DNULL;
			m_Yaw			= DNULL;
			m_Roll			= DNULL;
			m_hTarget		= DNULL;

            break;
        } 

        case MID_INITIALUPDATE:
        {
            // Tell the engine to update us as soon as possible.
            GetServerDE()->SetNextUpdate(m_hObject, 0.001f);

            // Give us some dimensions.
            dims.x = 16.0f;
            dims.y = 27.2f;
            dims.z = 10.3f;
            GetServerDE()->SetObjectDims(m_hObject, &dims); 

            // set up some shortcuts to animation indexes

            m_WalkIndex  = GetServerDE()->GetAnimIndex(m_hObject, "run");
            m_StandIndex = GetServerDE()->GetAnimIndex(m_hObject, "idle1");
			m_JumpIndex  = GetServerDE()->GetAnimIndex(m_hObject, "jump");

            m_CurrentAnimationIndex=m_StandIndex;
            GetServerDE()->SetModelAnimation(m_hObject,m_StandIndex);

            break;
        } // End MID_INITIALUPDATE

        case MID_UPDATE:
        {
            int iPlan=GetServerDE()->IntRandom(0, 100);
            static DVector LastTargetPosition;
            IntersectQuery Query;
            IntersectInfo Info;
            DVector DVECTemp;

			VEC_INIT(LastTargetPosition);

            // What's the bad guy's current vectors and speed?
            GetServerDE()->GetVelocity(m_hObject, &velocity);
            GetServerDE()->GetObjectRotation(m_hObject, &rotation);
            GetServerDE()->GetRotationVectors(&rotation, &up, &right, &forward);
            GetServerDE()->GetObjectPos(m_hObject, &position);

            switch (m_ActionState)
            {
            case WANDER:
                GetServerDE()->SetNextUpdate(m_hObject, 0.5f);
				m_CurrentAnimationIndex = m_WalkIndex;

                // if iPlan is in the range of 0 to 49 (50% chance) Walk in a straight line
                if (iPlan<50)
                {
					// Is there something in front of us?  if not, turn!
					if (!OnSolid(position,forward))
					{
						iPlan = 65;
					}
					else
					{
						// If we hit a wall, step back and try again
						Query.m_From=position;
						VEC_MULSCALAR(DVECTemp, forward, 50);
						VEC_ADD(Query.m_To, position, DVECTemp); 
						Query.m_Flags=INTERSECT_OBJECTS|IGNORE_NONSOLID;
						Query.m_FilterFn=NULL;
                        
						if (GetServerDE()->IntersectSegment(&Query, &Info))
							iPlan+=50;
						else
						{
							// The Character doesn't nee to Run.  Walk now
							VEC_MULSCALAR(temp, forward, moveSpeed/4);
							VEC_ADD(velocity, velocity, temp);						
						}
					}
                }

                // if iPlan is in the range of 50 to 59 (10% chance) Stand still
                if (iPlan>=50&&iPlan<60)            
                {
                    VEC_MULSCALAR(temp, forward, 0.0f);
                    VEC_ADD(velocity, velocity, temp);
					m_CurrentAnimationIndex = m_StandIndex;
					m_ActionState = STAND;
                }

                //if iPlan is in the range of 60 to 84 (24% chance) Turn Left
                if (iPlan>=60&&iPlan<85)            
                {
                    m_Yaw-=0.1f;
                    GetServerDE()->SetupEuler(&rotation, m_Pitch, m_Yaw, m_Roll);
					m_CurrentAnimationIndex = m_StandIndex;
                }

                // if iPlan is in the range of 85 to 100 (15% chance) Turn Right
                if (iPlan>=85)                      
                {
                    m_Yaw+=0.1f;
                    GetServerDE()->SetupEuler(&rotation, m_Pitch, m_Yaw, m_Roll);
					m_CurrentAnimationIndex = m_StandIndex;
                }

                AssignTarget(LocateOpponent(position, forward));
                break;

            case HUNT:
				// We've seen the player, now we are hunting for it
                GetServerDE()->SetNextUpdate(m_hObject, 0.25f);
				m_CurrentAnimationIndex = m_WalkIndex;

                // if iPlan is in the range of 0 to 49 (50% chance) Walk Straight
                if (iPlan<50)
                {
					// Is there something in front of us?  if not, turn!
					if (!OnSolid(position,forward))
					{
						iPlan = 65;
					}
					else
					{
						//Were about to walk into a wall.
						Query.m_From=position;
						VEC_MULSCALAR(DVECTemp, forward, 50);
						VEC_ADD(Query.m_To, position, DVECTemp); 
						Query.m_Flags=INTERSECT_OBJECTS|IGNORE_NONSOLID;
						Query.m_FilterFn=NULL;
                        
						if (GetServerDE()->IntersectSegment(&Query, &Info))
							iPlan+=50;
						else
						{
							// Need to carefully look for the player.  Move a little slower
							VEC_MULSCALAR(temp, forward, moveSpeed/2);
							VEC_ADD(velocity, velocity, temp);
						}
					}
                }

                // if iPlan is in the range of 50 to 59 (10% chance) Stop
                if (iPlan>=50&&iPlan<60)            
                {
                    VEC_MULSCALAR(temp, forward, 0.0f);
                    VEC_ADD(velocity, velocity, temp);
					m_CurrentAnimationIndex = m_StandIndex;
					m_ActionState = WANDER;
                }

                // if iPlan is in the range of 60 to 84 (24% chance) Turn Left
                if (iPlan>=60&&iPlan<85)            
                {
                    m_Yaw-=0.1f;
                    GetServerDE()->SetupEuler(&rotation, m_Pitch, m_Yaw, m_Roll);
					m_CurrentAnimationIndex = m_StandIndex;
                }

                // if iPlan is in the range of 85 to 100 (15% chance) Turn Right
                if (iPlan>=85)                      
                {
                    m_Yaw+=0.1f;
                    GetServerDE()->SetupEuler(&rotation, m_Pitch, m_Yaw, m_Roll);
					m_CurrentAnimationIndex = m_StandIndex;
                }

                // Find a target and try to touch it.
                AssignTarget(LocateOpponent(position, forward));
                break;

			// Just waiting around for something
            case STAND:
                AssignTarget(LocateOpponent(position, forward));

                break;

            
            default:
				// We've gotten here, but we really shouldn't have.
                m_ActionState=STAND;
                GetServerDE()->SetNextUpdate(m_hObject, 0.10f);
				m_CurrentAnimationIndex = m_StandIndex;
				break;
            
            }

            // Update the object's velocity.
            GetServerDE()->SetObjectRotation(m_hObject, &rotation);         
            GetServerDE()->SetVelocity(m_hObject, &velocity);
            // set the model to the right animation
            GetServerDE()->SetModelAnimation(m_hObject, m_CurrentAnimationIndex);

            break;
        }
    }
    
    // Pass the message on down.
    return BaseClass::EngineMessageFn(messageID, pData, lData);
}

DDWORD DarkClaw::ObjectMessageFn(HOBJECT hSender, DDWORD messageID, HMESSAGEREAD hRead)
{

	return BaseClass::ObjectMessageFn(hSender, messageID, hRead);
}

// Let the 'DarkClaw' Object look and find the Player
HOBJECT DarkClaw::LocateOpponent(DVector position, DVector forward)
{
    ObjectList *pObjectList=DNULL;
    IntersectQuery Query;
    IntersectInfo Info;
    ObjectLink *lpTestTarget=DNULL;
    HOBJECT hPlayerObject=DNULL;

    // Find the Player
    pObjectList=GetServerDE()->FindNamedObjects("Player");
    if (DNULL!=pObjectList->m_pFirstLink)
    {
        DVector VectorToPlayer;

        lpTestTarget=pObjectList->m_pFirstLink;
        hPlayerObject=lpTestTarget->m_hObject;

        //Parse the list
        while (lpTestTarget!=DNULL)
        {
            DVector PlayerPosition;

            GetServerDE()->GetObjectPos(hPlayerObject, &PlayerPosition);
            
            //Where do we have to look to find the player?
            VEC_SUB(VectorToPlayer, position, PlayerPosition);
            
            //Can we see the player?
            Query.m_Flags=IGNORE_NONSOLID;
            Query.m_From=position;
            Query.m_To=PlayerPosition;
            if ((!GetServerDE()->IntersectSegment(&Query, &Info))&&(VEC_DOT(VectorToPlayer, forward)<0))
            {
                // We've found the target.
                break;
            }

            // Can't see the target, check for another Potential target
            lpTestTarget=lpTestTarget->m_pNext;
            if (lpTestTarget==DNULL) hPlayerObject=DNULL;
            else hPlayerObject=lpTestTarget->m_hObject;
        }
    }   
    GetServerDE()->RelinquishList(pObjectList);

    // Now we have either a DNULL or a valid player object in hPlayerObject
    // We should return that.
    return hPlayerObject;
}

bool DarkClaw::OnSolid(DVector CurrentPos, DVector Forward)
{
	bool bReturn = true;
	DVector PositionInFront,Delta;
	IntersectQuery Query;
	IntersectInfo Info;

	VEC_MULSCALAR(Delta,Forward,20);
	VEC_ADD(PositionInFront,CurrentPos,Delta);
    
	//Can we see the the floor from a spot just in front of us?
    Query.m_Flags=IGNORE_NONSOLID;
    Query.m_From=PositionInFront;

	// Now we put the position to where we test down a unit or two
	PositionInFront.y -= 10;

    Query.m_To=PositionInFront;
    if (!GetServerDE()->IntersectSegment(&Query, &Info))
    {
        // We shouldn't move here.
		bReturn = false;
    }


/*	CollisionInfo Info;

    GetServerDE()->GetStandingOn(m_hObject, &Info);
    if (Info.m_hObject!=DNULL) return true;
*/
    return bReturn;
}


void DarkClaw::AssignTarget(HOBJECT hObject)
{
    if (hObject!=DNULL)
    {
        GetServerDE()->BreakInterObjectLink(m_hObject, m_hTarget);
        m_hTarget=hObject;
        m_ActionState=HUNT;
        GetServerDE()->CreateInterObjectLink(m_hObject, m_hTarget);
        GetServerDE()->SetNextUpdate(m_hObject, 0.001f);
    }
    else
        GetServerDE()->SetNextUpdate(m_hObject, 1.0f);
}

// Expose the DarkClaw class for LithTech.
BEGIN_CLASS(DarkClaw)
END_CLASS_DEFAULT(DarkClaw, BaseClass, NULL, NULL)


