// --------------------------------
// File    : LineSystem.cpp
// Function: Level Object Implementation used as the property 
//         : placeholder for the Line System effect.
// --------------------------------

#include <stdio.h>

#include "de_server.h"
#include "LineSystem.h"
#include "messageIDs.h"

// Our simple Line System source object

DDWORD LineSystem::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
	switch(messageID)
	{
		// This is where the variables from the level editor are read.
		case MID_PRECREATE:
		{
			((ObjectCreateStruct*)pData)->m_Flags = FLAG_FORCECLIENTUPDATE;

			// As a general rule, it's best to use GetPropGeneric instead of
			// GetProp<type>, tempting as the shortcut may be.  Generic props
			// are more flexible.
			GenericProp gProp;

			if (GetServerDE()->GetPropGeneric("StartPoint",&gProp) == DE_OK)
				m_vStart = gProp.m_Vec;
			else
				VEC_SET(m_vStart,1.0f,1.0f,1.0f);

			if (GetServerDE()->GetPropGeneric("EndPoint",&gProp) == DE_OK)
				m_vEnd = gProp.m_Vec;
			else
				VEC_SET(m_vStart,1.0f,1.0f,1.0f);

			if (GetServerDE()->GetPropGeneric("Color1",&gProp) == DE_OK)
				m_vColor1 = gProp.m_Color;
			else
				VEC_SET(m_vColor1,0.0f,0.0f,255.0f);


			break;
		}

		// Our initial update.. 
		// Setup the special effect message.
		case MID_INITIALUPDATE:
		{
			HMESSAGEWRITE hWrite;

			// Send all our relevant data to the client.
			hWrite = GetServerDE()->StartSpecialEffectMessage(this);

			// Set our message type
			GetServerDE()->WriteToMessageByte(hWrite, MSG_ADD_LINESYSTEM);

			// And our effect-specific stuff.
			GetServerDE()->WriteToMessageVector(hWrite, &m_vStart);
			GetServerDE()->WriteToMessageVector(hWrite, &m_vEnd);
			GetServerDE()->WriteToMessageVector(hWrite, &m_vColor1);
			GetServerDE()->EndMessage(hWrite);

			break;
		}
	}

	// Pass the message on down.
	return BaseClass::EngineMessageFn(messageID, pData, lData);
}

// Class definition.
BEGIN_CLASS(LineSystem)
ADD_VECTORPROP_VAL(StartPoint,0.0f,0.0f,0.0f)
ADD_VECTORPROP_VAL(EndPoint,0.0f,0.0f,0.0f)
ADD_COLORPROP(Color1,0.0f,0.0f,0.0f)
END_CLASS_DEFAULT(LineSystem, BaseClass, NULL, NULL)
