// --------------------------------
// File    : ParticleSystem.cpp
// Function: Level Object Implementation used as the property 
//         : placeholder for the Particle System effect.
//         : Heavily based on the Particle System Presented by
//         : Jeremy Blackman of Monolith.
// --------------------------------

#include <stdio.h>

#include "de_server.h"
#include "ParticleSystem.h"
#include "messageIDs.h"

// // The Particle System's Message Handling Function

DDWORD ParticleSystem::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
    HMESSAGEWRITE hWrite;

    switch(messageID)
    {
        // This is where the variables from the level editor are read.
        case MID_PRECREATE:
        {
			((ObjectCreateStruct*)pData)->m_Flags = FLAG_FORCECLIENTUPDATE;

            GenericProp gProp;

			// Get the radius of the light the particle system creates
            if (GetServerDE()->GetPropGeneric("LightRadius",&gProp) == DE_OK)
                LightRadius = gProp.m_Float;
            else
                LightRadius = 200.0f;

			// Get the light duration (how long the light 'lives')
            if (GetServerDE()->GetPropGeneric("LightDuration",&gProp) == DE_OK)
                LightDuration = gProp.m_Float;
            else
                LightDuration = 3.0f;

            break;
        }

        // Our initial update.. 
        // Setup the special effect message.
        case MID_INITIALUPDATE:
        {
            // Start the Special Effect Message
            hWrite = GetServerDE()->StartSpecialEffectMessage(this);

			// We want to create a Particle System
  
			GetServerDE()->WriteToMessageByte(hWrite, MSG_ADD_PARTICLESYSTEM);

			// Now send the data
            GetServerDE()->WriteToMessageFloat(hWrite, LightRadius);
            GetServerDE()->WriteToMessageFloat(hWrite, LightDuration);

            GetServerDE()->EndMessage(hWrite);

			break;
		}

    }

    // Pass the message on down to our parent.
    return BaseClass::EngineMessageFn(messageID, pData, lData);;
}

// Class definition.
BEGIN_CLASS(ParticleSystem)
    ADD_REALPROP(LightRadius,200.0f)
    ADD_REALPROP(LightDuration,3.0f)
END_CLASS_DEFAULT(ParticleSystem, BaseClass, NULL, NULL)
