// --------------------------------
// File    : CoinObject.cpp
// Function: Implementation of a game peice representing a coing piece the Player has to collect
// --------------------------------

#include <stdio.h>

#include "de_server.h"
#include "CoinObject.h"
#include "messageIDs.h"

// Our simple Line System source object

DDWORD Coin::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
    DRotation rotation;
    DVector dims;

	switch(messageID)
	{
		// This is where the variables from the level editor are read.
		case MID_PRECREATE:
		{
            // The coin should be visible and be notified when it's touched.
            ((ObjectCreateStruct*)pData)->m_ObjectType = OT_MODEL;
            ((ObjectCreateStruct*)pData)->m_Flags |= FLAG_VISIBLE | FLAG_TOUCH_NOTIFY | FLAG_FULLPOSITIONRES;

            // Now we load the model we want to use.
            strcpy(((ObjectCreateStruct*)pData)->m_Filename, "models\\coin.abc");


			// As a general rule, it's best to use GetPropGeneric instead of
			// GetProp<type>, tempting as the shortcut may be.  Generic props
			// are more flexible.
			GenericProp gProp;


			// Get the color this coin should have (From the Editor)
			if (GetServerDE()->GetPropGeneric("Color1",&gProp) == DE_OK)
				m_vColor1 = gProp.m_Color;
			else
				VEC_SET(m_vColor1,0.0f,0.0f,255.0f);


			// Get the value this coin should have (From the Editor)
			if (GetServerDE()->GetPropGeneric("CoinValue",&gProp) == DE_OK)
				m_CoinValue = gProp.m_Long;
			else
				m_CoinValue = 1;
			break;
		}

		// Our initial update.. 
		// Since this is a game piece, we control it on the Server, not the client
		case MID_INITIALUPDATE:
		{
            // Tell the engine to update us as soon as possible.
            GetServerDE()->SetNextUpdate(m_hObject, 0.001f);

            // Give the coin some dimensions.
            dims.x = 16.0f;
            dims.y = 27.2f;
            dims.z = 5.0f;
            GetServerDE()->SetObjectDims(m_hObject, &dims); 

			// And Set the coin's color
			GetServerDE()->SetObjectColor(	m_hObject,
												m_vColor1.x/255.0f,
												m_vColor1.y/255.0f,
												m_vColor1.z/255.0f,
												1.0f
										);
			break;
		}

        case MID_UPDATE:
        {
            // Update the coin's rotation
            GetServerDE()->GetObjectRotation(m_hObject, &rotation);

			// turn the coin .01 radians to the left
			GetServerDE()->EulerRotateY(&rotation, -0.2f);

			// And now set the object's new rotation
			GetServerDE()->SetObjectRotation(m_hObject, &rotation);

            // Register for another update..
            GetServerDE()->SetNextUpdate(m_hObject, 0.001f);
            break;
        } // End MID_UPDATE

		case MID_TOUCHNOTIFY:
		{
			HMESSAGEWRITE hWrite;
			DVector Pos;

			// Send the point value of this coin to the Player
			hWrite = GetServerDE()->StartMessageToObject(this,(HOBJECT)pData,20);

			GetServerDE()->WriteToMessageByte(hWrite,m_CoinValue);

			GetServerDE()->EndMessage(hWrite);

			// play a 'ting!' sound, indicating we have picked up the object
			PlaySound("sounds\\coin.wav");

			// Mark the current object Ready for deletion.
			GetServerDE()->RemoveObject(m_hObject);

			break;
		}

	}

	// Pass the message on down.
	return BaseClass::EngineMessageFn(messageID, pData, lData);
}

void Coin::PlaySound(char *szSoundName)
{
    CServerDE* pServerDE = GetServerDE();

    if (!pServerDE) return;

    //pServerDE->PlaySoundLocal(&playSoundInfo);
	pServerDE->PlaySoundLocal(szSoundName,0,100);

}

// Class definition.
BEGIN_CLASS(Coin)
ADD_COLORPROP(Color1,0.0f,0.0f,0.0f)
ADD_LONGINTPROP(CoinValue,1)
END_CLASS_DEFAULT(Coin, BaseClass, NULL, NULL)
