

#include "de_server.h"
#include "player.h"



// Command IDs (mapped to keys in autoexec.cfg).
#define COMMAND_FORWARD		0
#define COMMAND_BACKWARD	1
#define COMMAND_LEFT		10
#define COMMAND_RIGHT		11
#define COMMAND_STRAFE		9
#define COMMAND_SPEED		7
#define COMMAND_JUMP		25

DDWORD Player::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
    float moveSpeed = 45.0f;
    float terminalVelocity = 45.0f;
    DVector velocity, forward, right, up, temp;
    DRotation rotation;
    ObjectCreateStruct *pObjectCreateStruct;
    DVector dims;
	HMESSAGEWRITE hWrite;
	bool MovementOccurred;


    // See serverobj_de.h (near line 100) for descriptions of the messages.
    switch(messageID)
    {
        case MID_PRECREATE:
        {
            pObjectCreateStruct = (ObjectCreateStruct*)pData;
            
            // The model must be solid, have gravity applied to it, visible, ... blah blah blah
            pObjectCreateStruct->m_ObjectType = OT_MODEL;
            pObjectCreateStruct->m_Flags |= FLAG_SOLID | FLAG_GRAVITY | FLAG_VISIBLE | FLAG_TOUCH_NOTIFY | FLAG_SHADOW | FLAG_STAIRSTEP | FLAG_FULLPOSITIONRES;
            VEC_SET(pObjectCreateStruct->m_Pos,0.0f,140.0f,0.0f);

            // Now we load the model we want to use.
            strcpy(pObjectCreateStruct->m_Filename, "models\\claw.abc");
            strcpy(pObjectCreateStruct->m_SkinName, "textures\\claw.dtx");
			strcpy(pObjectCreateStruct->m_Name, "Player");
			m_bPlayerInAir = false;

			// And initialize the player's score
			m_Score = 0;
            break;
        } 

        case MID_INITIALUPDATE:
        {
            // Tell the engine to update us as soon as possible.
            GetServerDE()->SetNextUpdate(m_hObject, 0.001f);

			ReStartPlayer();

            // Give us some dimensions.
            dims.x = 16.0f;
            dims.y = 27.2f;
            dims.z = 10.3f;
            GetServerDE()->SetObjectDims(m_hObject, &dims); 

            // set up some shortcuts to animation indexes

            m_WalkIndex  = GetServerDE()->GetAnimIndex(m_hObject, "run");
            m_StandIndex = GetServerDE()->GetAnimIndex(m_hObject, "idle1");
			m_JumpIndex  = GetServerDE()->GetAnimIndex(m_hObject, "jump");

            m_CurrentAnimationIndex=m_StandIndex;
            GetServerDE()->SetModelAnimation(m_hObject,m_StandIndex);

            break;
        } // End MID_INITIALUPDATE

        case MID_UPDATE:
        {
			switch (m_ePlayerState)
			{
			case PLAYER_ALIVE:
				{
					// Check what commands are on and apply movement.
					GetServerDE()->GetVelocity(m_hObject, &velocity);
					GetServerDE()->GetObjectRotation(m_hObject, &rotation);
					GetServerDE()->GetRotationVectors(&rotation, &up, &right, &forward);
					MovementOccurred = false;

					if (m_hClient)
					{
						// Is the player in-air?
						if (m_bPlayerInAir)
						{
							if (OnSolid())
							{
								m_bPlayerInAir = false;
								dims.x = 16.0f;
								dims.y = 27.2f;
								dims.z = 10.3f;
								GetServerDE()->SetObjectDims2(m_hObject, &dims); 
								m_CurrentAnimationIndex=m_StandIndex;
							}
						}
						else
						{
							m_CurrentAnimationIndex=m_StandIndex;

							if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_FORWARD))
							{
								m_CurrentAnimationIndex=m_WalkIndex;
								VEC_MULSCALAR(temp, forward, moveSpeed*2.0f);
								VEC_ADD(velocity, velocity, temp);
								MovementOccurred = true;
							}

							if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_BACKWARD))
							{
								m_CurrentAnimationIndex=m_WalkIndex;
								VEC_MULSCALAR(temp, forward, -moveSpeed);
								VEC_ADD(velocity, velocity, temp);
								MovementOccurred = true;
							}

							if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_LEFT))
							{
								// turn the character .01 radians to the left
								GetServerDE()->GetObjectRotation(m_hObject, &rotation);
								GetServerDE()->EulerRotateY(&rotation, -0.2f);
								GetServerDE()->SetObjectRotation(m_hObject, &rotation);         
								MovementOccurred = true;
							}

							if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_RIGHT)) 
							{
								// turn the character .01 radians to the right
								GetServerDE()->GetObjectRotation(m_hObject, &rotation);
								GetServerDE()->EulerRotateY(&rotation, 0.2f);
								GetServerDE()->SetObjectRotation(m_hObject, &rotation);         
								MovementOccurred = true;
							}

							if (!MovementOccurred)
							{
								// Slow down for when they aren't pressing keys.
								VEC_MULSCALAR(velocity, velocity, 0.2f);
								if(VEC_MAG(velocity) < 0.05f)
									VEC_SET(velocity, 0, 0, 0);
							}

							if(GetServerDE()->IsCommandOn(m_hClient, COMMAND_JUMP))
							{
								float fJumpVelocity;
								m_CurrentAnimationIndex = m_JumpIndex;
								fJumpVelocity=600;
								VEC_MULSCALAR(temp, up, fJumpVelocity);
								VEC_ADD(velocity, velocity, temp);
								m_bPlayerInAir = true;
								dims.x = 6.0f;
								dims.y = 6.0f;
								dims.z = 6.0f;
								GetServerDE()->SetObjectDims(m_hObject, &dims); 
							}
                
						}
					}
					break;
				}  //	PLAYER_ALIVE
				case PLAYER_DEAD:
				{
					// The body stays where it is
					GetServerDE()->SetObjectPos(m_hObject,&m_PlayersDeathSpot);

					// The body should be still
					m_CurrentAnimationIndex=m_StandIndex;

					// CountDown to re-animate player
					m_ReAnimate_Timer ++;

					if (m_ReAnimate_Timer > 30)
						ReStartPlayer();
				}
			}

            // set the model to the right animation
            GetServerDE()->SetModelAnimation(m_hObject, m_CurrentAnimationIndex);

            // Update the object's velocity.
            GetServerDE()->SetVelocity(m_hObject, &velocity);
            
            // Register for another update..
            GetServerDE()->SetNextUpdate(m_hObject, 0.001f);
            break;
        } // End MID_UPDATE
    }
    
    // Pass the message on down.
    return BaseClass::EngineMessageFn(messageID, pData, lData);
}

DDWORD Player::ObjectMessageFn(HOBJECT hSender, DDWORD messageID, HMESSAGEREAD hRead)
{
	long CoinValue;
	HMESSAGEWRITE hWrite;

	// If We've collected a coin, Let's update the player's score.
	if (messageID == 20)
	{
		CoinValue = (long)GetServerDE()->ReadFromMessageByte(hRead);
		m_Score += CoinValue;

		// Now send a message to the server telling it that there is a new score
		hWrite = GetServerDE()->StartMessage(m_hClient, 20);

		GetServerDE()->WriteToMessageDWord(hWrite,m_Score);
		GetServerDE()->EndMessage(hWrite);
	}

	// We've Just Died.  After a pause, restart the level
	if (messageID == 21)
	{

		// Ignore if the player is already dead
		if (m_ePlayerState != PLAYER_DEAD)
		{
			m_ePlayerState = PLAYER_DEAD;
			m_ReAnimate_Timer = 0;

			// Get the player's current Position.  He stays dead there
			GetServerDE()->GetObjectPos(m_hObject,&m_PlayersDeathSpot);

			// Now send a message to the server telling it that we just died
			hWrite = GetServerDE()->StartMessage(m_hClient, 21);

			GetServerDE()->EndMessage(hWrite);
		}	
	}

	return BaseClass::ObjectMessageFn(hSender, messageID, hRead);
}

bool Player::OnSolid()
{
	CollisionInfo Info;

    GetServerDE()->GetStandingOn(m_hObject, &Info);
    if (Info.m_hObject!=DNULL) return true;

    return false;
}

void Player::ReStartPlayer()
{
    // Find us a start point.
    HOBJECT hTemp;
    DBOOL bDone = DFALSE;

    hTemp = NULL;

    while ((hTemp = GetServerDE()->GetNextObject(hTemp)) && !bDone)
    {
        // Find our first startpoint.
        if(GetServerDE()->IsKindOf(	GetServerDE()->GetObjectClass(hTemp),
									GetServerDE()->GetClass("StartPoint")))
        {
            DVector vec;
            DRotation rot;

            bDone = DTRUE;
            GetServerDE()->GetObjectPos(hTemp,&vec);
            GetServerDE()->GetObjectRotation(hTemp,&rot);

            GetServerDE()->TeleportObject(m_hObject,&vec);
            GetServerDE()->SetObjectRotation(m_hObject,&rot);
        }
    }
	
	m_ePlayerState = PLAYER_ALIVE;
}

// Expose the Player class for LithTech.
BEGIN_CLASS(Player)
// Properties would go here if this were an object we wanted level
// designers to be able to edit them from within DirectEditor.
//	ADD_REALPROP(SomeNum, 6)
END_CLASS_DEFAULT_FLAGS(Player, BaseClass, NULL, NULL, CF_HIDDEN)