// --------------------------------
// File    : PolyGridSource.cpp
// Function: Level Object Implementation used as the property 
//         : placeholder for the Poly Grid System effect.
// --------------------------------

#include <stdio.h>

#include "de_server.h"
#include "polygridsource.h"
#include "messageIDs.h"

// Our simple PolyGrid source object

DDWORD PolyGridSource::EngineMessageFn(DDWORD messageID, void *pData, DFLOAT lData)
{
	switch(messageID)
	{
		// This is where the variables from the level editor are read.
		case MID_PRECREATE:
		{
			((ObjectCreateStruct*)pData)->m_Flags = FLAG_FORCECLIENTUPDATE;

			// As a general rule, it's best to use GetPropGeneric instead of
			// GetProp<type>, tempting as the shortcut may be.  Generic props
			// are more flexible.
			GenericProp gProp;

			if (GetServerDE()->GetPropGeneric("Dims",&gProp) == DE_OK)
				m_vDims = gProp.m_Vec;
			else
				VEC_SET(m_vDims,1.0f,1.0f,1.0f);


			if (GetServerDE()->GetPropGeneric("Color1",&gProp) == DE_OK)
				m_vColor1 = gProp.m_Color;
			else
				VEC_SET(m_vColor1,0.0f,0.0f,255.0f);


			if (GetServerDE()->GetPropGeneric("Color2",&gProp) == DE_OK)
				m_vColor2 = gProp.m_Color;
			else
				VEC_SET(m_vColor2,255.0f,255.0f,255.0f);


			if (GetServerDE()->GetPropGeneric("XScaleMin",&gProp) == DE_OK)
				m_fXScaleMin = gProp.m_Float;
			else
				m_fXScaleMin = 15.0f;


			if (GetServerDE()->GetPropGeneric("XScaleMax",&gProp) == DE_OK)
				m_fXScaleMax = gProp.m_Float;
			else
				m_fXScaleMax = 25.0f;


			if (GetServerDE()->GetPropGeneric("YScaleMin",&gProp) == DE_OK)
				m_fYScaleMin = gProp.m_Float;
			else
				m_fYScaleMin = 15.0f;


			if (GetServerDE()->GetPropGeneric("YScaleMax",&gProp) == DE_OK)
				m_fYScaleMax = gProp.m_Float;
			else
				m_fYScaleMax = 25.0f;


			if (GetServerDE()->GetPropGeneric("XScaleDuration",&gProp) == DE_OK)
				m_fXScaleDuration = gProp.m_Float;
			else
				m_fXScaleDuration = 10.0f;

			
			if (GetServerDE()->GetPropGeneric("YScaleDuration",&gProp) == DE_OK)
				m_fYScaleDuration = gProp.m_Float;
			else
				m_fYScaleDuration = 10.0f;


			if (GetServerDE()->GetPropGeneric("XPan",&gProp) == DE_OK)
				m_fXPan = gProp.m_Float;
			else
				m_fXPan = 0.0f;


			if (GetServerDE()->GetPropGeneric("YPan",&gProp) == DE_OK)
				m_fYPan = gProp.m_Float;
			else
				m_fYPan = 0.0f;


			if (GetServerDE()->GetPropGeneric("SpriteTexture",&gProp) == DE_OK)
				sprintf(m_szSurfaceSprite,"%s",gProp.m_String);
			else
				sprintf(m_szSurfaceSprite,"none");


			if (GetServerDE()->GetPropGeneric("Transparent",&gProp) == DE_OK)
				m_bTransparent = gProp.m_Bool;
			else
				m_bTransparent = DFALSE;


			if (GetServerDE()->GetPropGeneric("NumPolies",&gProp) == DE_OK)
				m_dwNumPolies = (DDWORD)gProp.m_Long;
			else
				m_dwNumPolies = 160;

			break;
		}

		// Our initial update.. 
		// Setup the special effect message.
		case MID_INITIALUPDATE:
		{
			HMESSAGEWRITE hWrite;
			DBYTE byTemp;

			if (m_bTransparent) byTemp = 1; else byTemp = 0;

			// Send all our relevant data to the client.
			hWrite = GetServerDE()->StartSpecialEffectMessage(this);

			// Set our message type
			GetServerDE()->WriteToMessageByte(hWrite, MSG_ADD_POLYGRID);

			// And our effect-specific stuff.
			GetServerDE()->WriteToMessageVector(hWrite, &m_vDims);
			GetServerDE()->WriteToMessageVector(hWrite, &m_vColor1);
			GetServerDE()->WriteToMessageVector(hWrite, &m_vColor2);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fXScaleMin);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fXScaleMax);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fYScaleMin);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fYScaleMax);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fXScaleDuration);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fYScaleDuration);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fXPan);
			GetServerDE()->WriteToMessageFloat(hWrite, m_fYPan);
			GetServerDE()->WriteToMessageByte(hWrite, byTemp);
			GetServerDE()->WriteToMessageDWord(hWrite,m_dwNumPolies);
			GetServerDE()->WriteToMessageString(hWrite, &m_szSurfaceSprite[0]);
			GetServerDE()->EndMessage(hWrite);

			break;
		}
	}

	// Pass the message on down.
	return BaseClass::EngineMessageFn(messageID, pData, lData);
}

// Class definition.
BEGIN_CLASS(PolyGridSource)
ADD_VECTORPROP_FLAG(Dims, PF_DIMS|PF_LOCALDIMS)
ADD_COLORPROP(Color1,0.0f,0.0f,0.0f)
ADD_COLORPROP(Color2,255.0f,255.0f,255.0f)
ADD_REALPROP(XScaleMin,15.0f)
ADD_REALPROP(XScaleMax,25.0f)
ADD_REALPROP(YScaleMin,15.0f)
ADD_REALPROP(YScaleMax,25.0f)
ADD_REALPROP(XScaleDuration,10.0f)
ADD_REALPROP(YScaleDuration,10.0f)
ADD_REALPROP(XPan,0.0f)
ADD_REALPROP(YPan,0.0f)
ADD_STRINGPROP(SpriteTexture,"none")
ADD_BOOLPROP(Transparent,DTRUE)
ADD_LONGINTPROP(NumPolies,160)
END_CLASS_DEFAULT(PolyGridSource, BaseClass, NULL, NULL)
