// Music System Implementation
//
// Details:
//     The implementation of the game's music system.
//
//

#include "basedefs_de.h"
#include "MusicSystem.h"

// cMusic Object
// Constructor for the Music System
cMusic::cMusic()
{
    m_MusicType = -1;
    m_CurrentSong = 0;
    m_SongCount = 0;
    m_MusicPaused = DTRUE;
	m_MusicVolume = 100;

}

//Initializer for the Music System
DBOOL cMusic::init(ClientDE * PtrToShell, char *szPathToDLS, char * szPlayListName, int  MusicType)
{
	char DLSFile[255],STYLEFile[255];
    DBOOL bReturnVal = DTRUE;

    // Store a pointer to the Client Shell
	m_ShellPtr = PtrToShell;

    strcpy(m_PlayList,szPlayListName);

    // Initialize the music system (based upon the MusicType)
    switch ( MusicType)
    {
        case 0 :                        // CD Audio Playback.
        {
            m_MusicType = 0;

            // Initialize the CD Music SubSystem
            if (!m_ShellPtr->InitMusic("cdaudio.dll"))
                bReturnVal = DFALSE;

			break;
        }
        case 1 :                        // IMA Music Playback
        {
            m_MusicType = 1;

            // Initialize the CD Music SubSystem
            if (!m_ShellPtr->InitMusic("ima.dll"))
                bReturnVal = DFALSE;

            // Now set the music Diorectory
            if ( !m_ShellPtr->SetMusicDirectory(szPathToDLS))
                bReturnVal = DFALSE;

			strcat(DLSFile,"\\shopop.dls");
			strcat(STYLEFile,"\\spopinit.sty");
			m_ShellPtr->InitInstruments(DLSFile,STYLEFile);

			break;
        }
    }

	// Set the volume of the music

	m_ShellPtr->SetMusicVolume(m_MusicVolume);
    return bReturnVal;
}

// Add a song to the playlist
DBOOL cMusic::AddSong(char * szSongTitle)
{
    DBOOL bReturnVal = DTRUE;

    if (!m_ShellPtr->AddSongToPlayList(m_PlayList,szSongTitle))
        bReturnVal = DFALSE;
    else
        m_SongCount++;

    return bReturnVal;
}

// Begin Playing a song
DBOOL cMusic::Begin()
{
    DBOOL bReturnVal = DTRUE;

    if ( !m_ShellPtr->PlayList(m_PlayList,DNULL,DTRUE, 1)) // Should be MUSIC_MEASURE))
        bReturnVal = DFALSE;
    else
    {
        m_MusicPaused = DFALSE;
        m_CurrentSong = 1;
    }
    return bReturnVal;
}

// Pause a song
DBOOL cMusic::Pause()
{
    DBOOL bReturnVal = DTRUE;

    if (m_MusicPaused )
    {
        if ( !m_ShellPtr->ResumeMusic())
            bReturnVal = DFALSE;
        else
            m_MusicPaused = DFALSE;
    }
    else
    {
        if ( !m_ShellPtr->PauseMusic())
            bReturnVal = DFALSE;
        else
            m_MusicPaused = DTRUE;
    }

    return bReturnVal;
}

// Stop Playing the song
DBOOL cMusic::Stop()
{
    DBOOL bReturnVal = DTRUE;

    m_ShellPtr->StopMusic(1); // Should be MUSIC_MEASURE)
    return bReturnVal;
}

// Skip the current song

DBOOL cMusic::Skip()
{
    DBOOL bReturnVal = DTRUE;
    return bReturnVal;
}

// Rewind to the last song
DBOOL cMusic::Back()
{
    DBOOL bReturnVal = DTRUE;
    return bReturnVal;
}

// No more songs are going to be played.
DBOOL cMusic::End()
{
    DBOOL bReturnVal = DTRUE;

	Stop();									// Stop the current Song
    m_ShellPtr->DestroyAllSongs();			// Destroy all songs in the Playlist
    m_ShellPtr->DeletePlayList(m_PlayList);	// Delete the PlayList

    return bReturnVal;
}
