// On-screen System Heads Up Display Header file
// 
// Details:
//     The Heads Up Display (HUD) for the game in progress
//     
//     

#include "basedefs_de.h"
#include "cHud.h"
#include "string.h"           

// CGameMessageWindow Object
// Details:
//     The Game Message Window system displays information into an 
//     on-screen window for Game Information Purposes as well as
//     Debug purposes
//     
/*   DRect   OuterRect,              // Outer Rectangle defines the Debug window
            InnerRect;               // Innder Rectangle defines the Text Area
     HSURFACE DrawTo;                // A drawing surface that is the Debug window
     int NumberofTextLines;          // The number of lines of text
     HDEFONT DisplayFont;            // The Debug font
     char StringDisplay[125][20];    // The text to display
     DVect TopCorner;                // Position of the top corner of the debug window
     int FontWidth,FontHeight;       // The width and the height of the font.
*/                                                                                                 

 // Default Constructor
 cGameMessageWindow::cGameMessageWindow()
 {
     // Define the outer window area
     OuterRect.top    = 431;
     OuterRect.left   = 180;
     OuterRect.bottom = 478;
     OuterRect.right  = 478;

     // Define the inner window area
     InnerRect.top    = 434;
     InnerRect.left   = 185;
     InnerRect.bottom = 476;
     InnerRect.right  = 475;

     // No Text Lines
     NumberofTextLines = 0;

     // The current Window Top left corner
     TopCorner.x = 0;
     TopCorner.y = 0;
 }

 cGameMessageWindow::~cGameMessageWindow()
 {
     // Clear the outer window area
     OuterRect.top = 0;
     OuterRect.left = 0;
     OuterRect.bottom = 0;
     OuterRect.right = 0;

     // Clear the outer window area
     InnerRect.top = 0;
     InnerRect.left = 0;
     InnerRect.bottom = 0;
     InnerRect.right = 0;

     // No Text Lines
     NumberofTextLines = 0;

     // Clear the current Window Top left corner
     TopCorner.x = 0;
     TopCorner.y = 0;

     //Finally, Remove reference to the Client
     ClientShellDEPtr = DNULL;
 }

 // Initialize the Debug system
 DBOOL cGameMessageWindow::Init(CClientShellDE * ClientShell, int left, int top)
 {
     // Tie the ClientShell to the Debug Object
     ClientShellDEPtr = ClientShell;

     if ( DNULL == ClientShellDEPtr )
         return DFALSE;

     // Create the Font we will be using for Debug
     DisplayFont = ClientShellDEPtr->GetClientDE()->CreateFont("Courier New", 6, 12, DFALSE, DFALSE, DFALSE);

     // Placement of the Debug Window
     TopCorner.x = left;
     TopCorner.y = top;

     // Font Colors
     ForeFontColor = ClientShellDEPtr->GetClientDE()->CreateColor(.1f, .9f, .1f, DFALSE);
     BackFontColor = ClientShellDEPtr->GetClientDE()->CreateColor(.0f, .0f, .0f, DTRUE);

	 // Add some strings to display
	 AddString("Game Message Window ");
	 AddString("Version 1.0.0a");
     return DTRUE;
 }

 DBOOL cGameMessageWindow::AddString(char *NewString)
 {
     // move the old strings down one item in the list, dropping the last item
     // off the list.
     if (NumberofTextLines > 0)
         for (int i = NumberofTextLines;i > 0; i--)
             strcpy(StringDisplay[i],StringDisplay[i-1]);

     strcpy(StringDisplay[0],NewString);

     if (NumberofTextLines < 8) NumberofTextLines++;

     return DTRUE;
 }

 DBOOL cGameMessageWindow::Render()
 {
     DRect DisplayStringRect;
     HSTRING DisplayString;

    // Now display the strings in the string list
    for (int i=1;i <= NumberofTextLines;i++)
    {
        // Create the HSTRING resource
        DisplayString = ClientShellDEPtr->GetClientDE()->CreateString(&StringDisplay[i-1][0]);

        // Calculate the String Rectangle size
        ClientShellDEPtr->
            GetClientDE()->
                GetStringDimensions(DisplayFont,DisplayString,
                                    &FontWidth,&FontHeight);

        DisplayStringRect.top    = (InnerRect.bottom + (int)TopCorner.y) -
                                 (FontHeight * i+1);
        DisplayStringRect.bottom = (InnerRect.bottom + (int)TopCorner.y) -
                                 (FontHeight * (i-1));
        DisplayStringRect.left   = InnerRect.left;
        DisplayStringRect.right  = InnerRect.right;

        ClientShellDEPtr->GetClientDE()->DrawStringToSurface(
                        ClientShellDEPtr->GetClientDE()->GetScreenSurface(),
                        DisplayFont,DisplayString,
                        &DisplayStringRect,ForeFontColor,BackFontColor
                        );
        // Free the String resource.
        ClientShellDEPtr->GetClientDE()->FreeString(DisplayString);
    }

    return DTRUE;
 }



// cHUD Object
// Default Constructor for the HUD
cHUD::cHUD()
{
	// Set the rectangles for the Action States
	// Later, implement things like the rectangles that define the 
	// Display elements, individual elements in the HUD graphic that 
	// you are looking for
}

//Initializer for the HUD
DBOOL cHUD::init(CClientShellDE * PtrToShell, char *szFilename, HSURFACE DisplaySurf, tDisplayStats * Display)
{
	// Set the Primary Display Surface
	PrimarySurface = DisplaySurf;

	// What does the display look like?
	DisplayStats = Display;

	// And a pointer to the Client Shell
	ShellPtr = PtrToShell;

	// Load the HUD Graphics
	hsHUDArt = ShellPtr->GetClientDE()->CreateSurfaceFromBitmap(szFilename);

	// Now we need to initialize the Game Message Window
	GameMessage.Init(PtrToShell, 3,1);

	return DTRUE;
}

// Render the HUD
DBOOL cHUD::Render()
{
	DRect DisplayRect;

	// HUD Posting time.  Check the Application state
	// Show the on-screen user information
	// Show the HUD Graphic Left Bottom
	// Set the Rectangle values 
	DisplayRect.left = 0;
	DisplayRect.top = 0;
	DisplayRect.right = 99;
	DisplayRect.bottom = 53;

	ShellPtr->GetClientDE()->DrawSurfaceToSurfaceTransparent(
									PrimarySurface,
									hsHUDArt,&DisplayRect,30,420,
									ShellPtr->GetClientDE()->CreateColor(1,0,1,DTRUE)
									);

	// Show the HUD Graphic Center Bottom 
	// Set the Rectangle values 
	DisplayRect.left = 0;
	DisplayRect.top = 110;
	DisplayRect.right = 299;
	DisplayRect.bottom = 159;

	ShellPtr->GetClientDE()->DrawSurfaceToSurfaceTransparent(
									PrimarySurface,
									hsHUDArt,&DisplayRect,180,429,
									ShellPtr->GetClientDE()->CreateColor(1,0,1,DTRUE)
									);
	// Show the HUD Graphic Right Bottom
	// Set the Rectangle values 
	DisplayRect.left = 0;
	DisplayRect.top = 54;
	DisplayRect.right = 99;
	DisplayRect.bottom = 110;

	ShellPtr->GetClientDE()->DrawSurfaceToSurfaceTransparent(
									PrimarySurface,
									hsHUDArt,&DisplayRect,530,420,
									ShellPtr->GetClientDE()->CreateColor(1,0,1,DTRUE)
									);

	// now finally render any text in the Game Message Window.
	GameMessage.Render();

	return DTRUE;
}
