// PlayerClient.CPP
// Description: The Player Object On the Client
// Function:	Creates a Player object on the client.  
//              This is something different than the Client on the
//              Server, as this is an object that we can use for
//				Grouping of Client Rendering functions (The HUD,
//				ChaseCam, etc).

#include "de_client.h"
#include "PlayerClient.h"

// --------------------------------------------------------------- //
// Command IDs (mapped to keys in autoexec.cfg).
// --------------------------------------------------------------- //

#define COMMAND_LEFT            10
#define COMMAND_RIGHT           11
#define COMMAND_LOOKUP          15
#define COMMAND_LOOKDOWN        16
#define COMMAND_QUIT            17
#define COMMAND_CAMERA_LEFT     20
#define COMMAND_CAMERA_RIGHT    21
#define COMMAND_CAMERA_TOGGLE   22
#define COMMAND_CAMERA_ZOOMIN   23
#define COMMAND_CAMERA_ZOOMOUT  24
#define COMMAND_CAMERA_RAISE    25
#define COMMAND_CAMERA_LOWER    26

PlayerClient::PlayerClient()
{
    m_hCamera = DNULL;
    m_pClient = DNULL;
}

PlayerClient::~PlayerClient()
{
}

// ----------------
// Init
//      Initializes the player with a pointer to the client.
// ----------------
void PlayerClient::Init(ClientDE *pClient)
{
	// Variable to store the display properties
	tDisplayStats TheDisplayStats;

    m_pClient = pClient;

    m_Pitch = 0.0f;
    m_Yaw = 0.0f;
    m_Roll = 0.0f;

    m_ChaseCam.Init(m_pClient);

	// Display Dimensions setup
	m_pClient->GetSurfaceDims(
             m_pClient->GetScreenSurface(),
             &(TheDisplayStats.ScreenWidth),
             &(TheDisplayStats.ScreenHeight)
             );
	// And get the Midpoints
	TheDisplayStats.MidScreenWidth = TheDisplayStats.ScreenWidth/2;
	TheDisplayStats.MidScreenHeight = TheDisplayStats.ScreenHeight/2;
	
	HUD = new cHUD();

	HUD->init(m_pClient,"interface//hud//hud.pcx",
			m_pClient->GetScreenSurface(),
			&TheDisplayStats);


}

// ----------------
// Update
//      Main update loop.
// ----------------
void PlayerClient::Update()
{
    HLOCALOBJ hClientObj;
    DFLOAT offsets[3];
    float radiansPerSec = 2.5f;

    // Get our client object
    hClientObj = m_pClient->GetClientObject();

    // If we're in the world, let's update!
    if(hClientObj)
    {
        // Handle our various Camera commands
        if (m_pClient->IsCommandOn(COMMAND_CAMERA_ZOOMIN))
        {
            m_ChaseCam.Zoom(-5.0f);
        }
        if (m_pClient->IsCommandOn(COMMAND_CAMERA_ZOOMOUT))
        {
            m_ChaseCam.Zoom(5.0f);
        }

		if (m_ChaseCam.GetCameraMode() == 1)
		{
			// Rotate the ChaseCam based on mouse movement
			m_pClient->GetAxisOffsets(offsets);

			m_cPitch = offsets[0] / 100.0f;
			m_cYaw = offsets[1] / 100.0f;

			if (m_cPitch > 0.01f)
				m_ChaseCam.UpdateYaw(radiansPerSec * m_pClient->GetFrameTime());
			if (m_cPitch < -0.01f)
				m_ChaseCam.UpdateYaw(radiansPerSec * m_pClient->GetFrameTime() * -1.0f);

			if (m_cYaw > 0.01f)
				m_ChaseCam.UpdatePitch(radiansPerSec * m_pClient->GetFrameTime() * -1.0f);
			if (m_cYaw < -0.01f)
				m_ChaseCam.UpdatePitch(radiansPerSec * m_pClient->GetFrameTime());
		}

        // Perform our camera update
        m_ChaseCam.Update();
    }
}

// ----------------
// OnCommandOn
//      Called from the clientshell OnCommandOn to handle any commands that
//      the clientshell didn't.  Really only handles the camera toggle.
// ----------------
void PlayerClient::OnCommandOn(int command)
{
	if (command == COMMAND_CAMERA_TOGGLE)
    {
        m_ChaseCam.NextCamera();
		switch(m_ChaseCam.GetCameraMode())
		{
			case 0:
				HUD->GameMessage.AddString("Chase Cam Behind.");
				break;
			case 1:
				HUD->GameMessage.AddString("Chase Cam Mouse Rotate");
				break;
			default:
				HUD->GameMessage.AddString("Undefined Camera Setting");
		}
		

    }

}
