
#include "Windows.h"
#include "stdio.h"
#include "sys\stat.h"
#include "de_client.h"
#include "PlayerClient.h"
#include "MusicSystem.h"
#include "sample_clientshell.h"



// --------------------------------------------------------------- //
// Command IDs (mapped to keys in autoexec.cfg).
// --------------------------------------------------------------- //

#define COMMAND_LEFT		10
#define COMMAND_RIGHT		11
#define COMMAND_LOOKUP		15
#define COMMAND_LOOKDOWN	16
#define COMMAND_QUIT		17

// Some Helper Functions

DBOOL FileExists (char* strPath)
{
    OFSTRUCT ofs;
    HFILE hFile = OpenFile (strPath, &ofs, OF_EXIST);
    if (hFile == HFILE_ERROR) return FALSE;

    return TRUE;
}

// --------------------------------------------------------------- //
// These functions create the client shell for LithTech.
// --------------------------------------------------------------- //

ClientShellDE* CreateClientShell(ClientDE *pClient)
{
    g_pClientDE = pClient;
    return ((ClientShellDE*)new CSampleClientShell);
}

void DeleteClientShell(ClientShellDE *pShell)
{
	delete ((CSampleClientShell*)pShell);
}

SETUP_CLIENTSHELL();



// --------------------------------------------------------------- //
// CSampleClientShell functions.
// --------------------------------------------------------------- //

CSampleClientShell::CSampleClientShell()
{
	m_hCamera = DNULL;
}

DRESULT CSampleClientShell::OnEngineInitialized(struct RMode_t *pMode, DGUID *pGuid)
{
	StartGameRequest request;
	InitSoundInfo pSoundInfo;

	// New addition
	if (GetClientDE()->SetRenderMode (pMode) != LT_OK)
	{
		GetClientDE()->Shutdown();
		return LT_ERROR;
	}

	// Initialize the sound
	INITSOUNDINFO_INIT(pSoundInfo);
	if (GetClientDE()->InitSound(&pSoundInfo) != LT_OK)
	{
		GetClientDE()->Shutdown();
		return LT_ERROR;
	}

     // Start playing the video
	PlayMovie(GetClientDE(),"movies\\logo.smk");

	return LT_OK;
}

void CSampleClientShell::OnEnterWorld()
{
	ObjectCreateStruct createStruct;


	INIT_OBJECTCREATESTRUCT(createStruct);
	createStruct.m_ObjectType = OT_CAMERA;
	m_hCamera = GetClientDE()->CreateObject(&createStruct);


    // Give our player a pointer to the client...
    m_Player.Init(GetClientDE());

    // Give our player a handle to the camera.
    m_Player.SetCamera(m_hCamera);

    // Init CD Audio.
    m_MusicTracker.init(GetClientDE(), DNULL, "GameWorld", 0);
    m_MusicTracker.AddSong("2");
    m_MusicTracker.Begin();
}


void CSampleClientShell::OnExitWorld()
{
	if(m_hCamera)
	{
		GetClientDE()->DeleteObject(m_hCamera);
		m_hCamera = DNULL;
		m_Player.SetCamera(DNULL);

	}
    m_MusicTracker.End();
}


void CSampleClientShell::Update()
{
    HCONSOLEVAR hVar;
    StartGameRequest request;

    // What the state of the current Game?
    // it could be :
    // GAME_PAUSED      The Game, she is paused
    // GAME_MOVIE       The Game, she is playing a movie
    // GAME_PLAYING     The Game, she is, how you say, rocking and rolling?
    // GAME_OVER        That's it man, Game Over man, Game over.
    // GAME_MENU        When we want to display a menu to the user
    switch (m_nGameState)
    {
        case GAME_PLAYING:
            UpdatePlayer();
        break;


        case GAME_MOVIE:
            UpdateMoviesState();
        break;


        case GAME_MENU :
        break;


        case GAME_PAUSED:
        break;

        case GAME_OVER:
        break;

        case GAME_LOADLEVEL:
            // Zero-out our request for safety.
            memset(&request,0,sizeof(request));

            // Look for the +runworld command-line parameter and start a world based on that.
            hVar = GetClientDE()->GetConsoleVar("runworld");
            sprintf(request.m_WorldName,"%s",GetClientDE()->GetVarValueString(hVar));
            request.m_Type = STARTGAME_NORMAL;
            GetClientDE()->StartGame(&request);

            m_nGameState = GAME_PLAYING;
        break;

        default : break;
    }
}

void CSampleClientShell::OnCommandOn(int command)
{
    if (m_nGameState != GAME_MOVIE)
        if(command == COMMAND_QUIT)
        {
            GetClientDE()->Shutdown();
        };

    //Pass the command to the Player Client Object
	m_Player.OnCommandOn(command);

}

void CSampleClientShell::OnKeyDown(int key, int rep)
{
    CClientDE* pClientDE = GetClientDE();
    if (!pClientDE) return;

    if (m_nGameState == GAME_MOVIE)
    {
        if (key == VK_ESCAPE && pClientDE->IsVideoPlaying() == VIDEO_PLAYING)
        {
            if (pClientDE->StopVideo() == DE_OK)
            {
                m_nGameState = GAME_LOADLEVEL;
            }
        }
        return;
    }
}

void CSampleClientShell::PlayMovie(CClientDE* pClientDE, char * szMovieName)
{
    DRESULT nResult;

    // attempt to play the movie
    if (FileExists(szMovieName))
        nResult = GetClientDE()->StartVideo (szMovieName, PLAYBACK_FULLSCREEN);
    else
        nResult = LT_ERROR;

    if (nResult != DE_OK)
        m_nGameState = GAME_LOADLEVEL;
    else
        m_nGameState = GAME_MOVIE;
    return;
}

void CSampleClientShell::UpdateMoviesState()
{
    CClientDE* pClientDE = GetClientDE();
    if (!pClientDE) return;

    GetClientDE()->ClearScreen(DNULL,CLEARSCREEN_SCREEN | CLEARSCREEN_RENDER);
    // If we're playing movies, see if the current one is finished...
    pClientDE->UpdateVideo();

    if (pClientDE->IsVideoPlaying() != VIDEO_PLAYING)
    {
        m_nGameState = GAME_LOADLEVEL;
    }

    GetClientDE()->FlipScreen(FLIPSCREEN_CANDRAWCONSOLE);
}

void CSampleClientShell::UpdatePlayer()
{
	HLOCALOBJ hCam;

    // Update our player
    m_Player.Update();

    hCam = m_Player.GetCamera();

    if (hCam) {
        // Clear the screen...
        GetClientDE()->ClearScreen(DNULL,
								CLEARSCREEN_SCREEN | CLEARSCREEN_RENDER);

        // Start the 3D Block...
        GetClientDE()->Start3D();

        // Render the screen
        GetClientDE()->RenderCamera(m_Player.GetCamera());

		GetClientDE()->StartOptimized2D();
		m_Player.HUD->Render();
		GetClientDE()->EndOptimized2D();

        // End the 3D Block...
        GetClientDE()->End3D();

        // ...and flip to the screen to have it visible.
        GetClientDE()->FlipScreen(FLIPSCREEN_CANDRAWCONSOLE);

    } else {
        GetClientDE()->CPrint("No Camera Available to Render!");
    }
}
